<?php
/**
 * @package WorldCup
 * @version $Id: wcp-admin.class.php 236599 2010-05-02 14:29:16Z ianhaycox $
 * @author ian
 * Copyright Ian Haycox, 2010
 *
 */

class WorldCupAdmin extends WorldCup {
	
	/*
	 * Constructor
	 */
	function __construct() {
		//global $wpdb;
		
		//$wpdb->show_errors(true);  - http://trac.buddypress.org/ticket/2361
		parent::__construct();
	}

	/**
	 * Activation hook.
	 * 
	 * Create database structure
	 */
	function activate() {
		
		require_once ABSPATH . 'wp-admin/includes/upgrade.php';
		
		global $wpdb;
		
		$installed_ver = get_option($this->prefix.'db_version');
		if ($installed_ver == '1.1' || $installed_ver == '1.0') {
			// Remove old database structure from beta version
			$wpdb->query("DROP TABLE {$wpdb->prefix}{$this->prefix}stage");
			$wpdb->query("DROP TABLE {$wpdb->prefix}{$this->prefix}team");
			$wpdb->query("DROP TABLE {$wpdb->prefix}{$this->prefix}venue");
			$wpdb->query("DROP TABLE {$wpdb->prefix}{$this->prefix}match");
			$wpdb->query("DROP TABLE {$wpdb->prefix}{$this->prefix}prediction");
		}
		
		// Plugin database table version
		$db_version = "1.2";
		
		$sql = "CREATE TABLE IF NOT EXISTS `{$wpdb->prefix}{$this->prefix}match` (
		  `match_id` int(11) NOT NULL AUTO_INCREMENT,
		  `match_no` int(11) NOT NULL,
		  `kickoff` datetime NOT NULL,
		  `home_team_id` int(11) NOT NULL,
		  `away_team_id` int(11) NOT NULL,
		  `home_goals` int(11) NOT NULL,
		  `away_goals` int(11) NOT NULL,
		  `home_penalties` int(11) NOT NULL,
		  `away_penalties` int(11) NOT NULL,
		  `venue_id` int(11) NOT NULL,
		  `is_result` BOOL NOT NULL DEFAULT '0', 
		  `extra_time` BOOL NOT NULL DEFAULT '0', 
		  `stage_id` int(11) NOT NULL,
		  `wwhen` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
		  PRIMARY KEY (`match_id`)
		)";
		$wpdb->query($sql);
		
		
		$sql = "CREATE TABLE IF NOT EXISTS `{$wpdb->prefix}{$this->prefix}prediction` (
		  `prediction_id` int(11) NOT NULL AUTO_INCREMENT,
		  `user_id` bigint(20) NOT NULL,
		  `match_id` int(11) NOT NULL,
		  `home_goals` int(11) NOT NULL,
		  `away_goals` int(11) NOT NULL,
		  `home_penalties` int(11) NOT NULL,
		  `away_penalties` int(11) NOT NULL,
		  `points` int(11) NOT NULL,
		  `wwhen` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
		  PRIMARY KEY (`prediction_id`)
		)";
		$wpdb->query($sql);
		
		
		$sql = "CREATE TABLE IF NOT EXISTS `{$wpdb->prefix}{$this->prefix}stage` (
		  `stage_id` int(11) NOT NULL AUTO_INCREMENT,
		  `stage_name` varchar(32) NOT NULL,
		  `is_group` tinyint(1) NOT NULL,
		  `sort_order` int(11) NOT NULL,
		  `wwhen` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
		  PRIMARY KEY (`stage_id`)
		)";
		$wpdb->query($sql);
		
		$sql = "CREATE TABLE IF NOT EXISTS `{$wpdb->prefix}{$this->prefix}team` (
		  `team_id` int(20) NOT NULL AUTO_INCREMENT,
		  `name` varchar(64) NOT NULL,
		  `country` char(3) NOT NULL,
		  `team_url` varchar(255) NOT NULL,
		  `wwhen` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
		  PRIMARY KEY (`team_id`)
		)";
		$wpdb->query($sql);
		
		$sql = "CREATE TABLE IF NOT EXISTS `{$wpdb->prefix}{$this->prefix}venue` (
		  `venue_id` int(11) NOT NULL AUTO_INCREMENT,
		  `venue_name` varchar(64) NOT NULL,
		  `venue_url` varchar(255) NOT NULL,
		  `stadium` varchar(64) NOT NULL,
		  `wwhen` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
		  PRIMARY KEY (`venue_id`)
		)";
		$wpdb->query($sql);
		
		// Installed plugin database table version
		$installed_ver = get_option($this->prefix.'db_version');
		// If the database has changed, update the structure while preserving data
		if (empty($installed_ver) || $db_version != $installed_ver) {
			
			update_option($this->prefix.'db_version', '1.2');
		}
 		update_option($this->prefix.'db_version', $db_version);
		
 		/*
 		 * Try to calculate the time difference between this server and South Africa
 		 */
 		add_option($this->prefix.'tz_diff', $this->getRSADiff());
 		update_option($this->prefix.'promo_link', 1);
 		
		/*
		 * Set Capabilities
		*/
		$role = get_role('administrator');
		$role->add_cap($this->prefix.'manager');	// Can manage players, teams etc.
		
		$role = get_role('editor');
		$role->add_cap($this->prefix.'manager');
		
		return true;
	}
	
	/**
	 * Deactivation hook.
	 */
	function deactivate() {
		return true;
	}
	
	
	function uninstall() {
		global $wpdb;
		
		$wpdb->query("DROP TABLE IF EXISTS {$wpdb->prefix}{$this->prefix}stage");
		$wpdb->query("DROP TABLE IF EXISTS {$wpdb->prefix}{$this->prefix}team");
		$wpdb->query("DROP TABLE IF EXISTS {$wpdb->prefix}{$this->prefix}venue");
		$wpdb->query("DROP TABLE IF EXISTS {$wpdb->prefix}{$this->prefix}match");
		$wpdb->query("DROP TABLE IF EXISTS {$wpdb->prefix}{$this->prefix}prediction");
		
		delete_option( $this->prefix.'db_version' );
		delete_option( $this->prefix.'promo_link' );
		delete_option( $this->prefix.'group_stats' );
		delete_option( $this->prefix.'tz_diff' );
		delete_option( $this->prefix.'tz_msg' );
		delete_option( $this->prefix.'scoring' );
		
		$roles = array("subscriber", "contributor", "author", "editor", "administrator");
		foreach ($roles as $role) {
			$arole = get_role($role);
			$arole->remove_cap($this->prefix.'manager');
		}
	}
	
	/**
	 * Create admin menu
	 */
	function admin_menu() {
				
		require_once(dirname(__FILE__).'/wcp-menu.class.php');
		$menu = new WorldCupMenu();
		$scoring = new WorldCupScoring();
		$overview = new WorldCupOverview();
		
		add_menu_page(__('World Cup Menu', WCP_TD), __('World Cup', WCP_TD), $this->prefix.'manager', $this->prefix.'menu', array($overview, 'overview'), WP_PLUGIN_URL.'/'.WCP_TD .'/images/football.png');
		add_submenu_page($this->prefix.'menu' ,__('Overview', WCP_TD), __('Overview', WCP_TD), $this->prefix.'manager', $this->prefix.'menu' , array($overview, 'overview'));
		add_submenu_page($this->prefix.'menu' ,__('Teams and Matches', WCP_TD), __('Teams and Matches', WCP_TD), $this->prefix.'manager', $this->prefix.'config' , array($menu, 'menu'));
		add_submenu_page($this->prefix.'menu' ,__('Predictions', WCP_TD), __('Predictions', WCP_TD), $this->prefix.'manager', $this->prefix.'predictions' , array($scoring, 'menu'));
		add_submenu_page($this->prefix.'menu' ,__('Help', WCP_TD), __('Help', WCP_TD), $this->prefix.'manager', $this->prefix.'help' , array($this, 'help'));
	}
	
	/**
	 * Style sheet for admin functions
	 */
	function admin_print_styles() {
?>		
<link type="text/css" rel="stylesheet" href="<?php echo WP_PLUGIN_URL . '/' . WCP_TD; ?>/css/style.css" />
<link type="text/css" rel="stylesheet" href="<?php echo WP_PLUGIN_URL . '/' . WCP_TD; ?>/css/admin-style.css" />
<link type="text/css" rel="stylesheet" href="<?php echo WP_PLUGIN_URL . '/' . WCP_TD; ?>/css/jquery-ui.css" />
<?php		
		wp_admin_css( 'css/dashboard' );
	}

	/**
	 * Javascript for admin functions
	 */
	function admin_print_scripts() {
		wp_enqueue_script($this->prefix.'admin_js', WP_PLUGIN_URL . '/' . WCP_TD . '/js/wcp-admin.js', array( 'jquery', 'jquery-ui-tabs', 'jquery-ui-dialog' ));
		wp_enqueue_script( 'postbox' );
	}
	
	/**
	 * Init hook
	 */
	function admin_init() {
		
		register_setting( $this->prefix.'option-group', $this->prefix.'tz_diff', array($this, 'valid_tz_diff') );
		register_setting( $this->prefix.'option-group', $this->prefix.'promo_link');
	}
	
	/*
	 * Validate option setting
	 */
	function valid_tz_diff($input) {
		
		if (!$this->is_hhmm($input)) {
			update_option($this->prefix.'tz_msg',"<div class='message error'><p><strong>".__('Timezone difference must be in [+/-]HH:MM format, e.g. +03:00.', WCP_TD)."</strong></p></div");
			return $this->getRSADiff();  // default value
		} else {
			update_option($this->prefix.'tz_msg',"<div class='message updated fade'><p><strong>".__('Timezone difference updated', WCP_TD)."</strong></p></div");
		}
		return $input;
	}
	
	function settings() {
		
		$msg = get_option($this->prefix.'tz_msg','');
		if (!empty($msg)) {
			echo $msg;
			update_option($this->prefix.'tz_msg','');
		}
?>		
	<div class="wrap">
	
	<table style="width:100%" class="updated fade">
	<tr>
		<td><?php _e('Server local time', WCP_TD); ?></td><td><?php echo $this->getServerDateTime(); ?></td>
	</tr>
	<tr>
		<td><?php _e('South African Time SAST', WCP_TD); ?></td><td><?php echo $this->getRSADateTime(); ?></td>
	</tr>
	<tr>
		<td><?php _e('Calculated Difference', WCP_TD); ?></td><td><?php echo $this->getRSADiff(); ?></td>
	</tr>
	</table>
	<p> <?php _e('Ensure this timezone difference is correct or change it below.', WCP_TD); ?></p>
	<p><a href="http://www.google.com/#hl=en&source=hp&q=time+in+south+africa"><?php _e('South African Time SAST', WCP_TD); ?></a></p>
	
	<form method="post" action="options.php">
	
	<?php settings_fields( $this->prefix.'option-group' );?>
	
	<table class="form-table">
	
	<tr valign="top">
	<th scope="row" style="width:300px"><?php _e('Add +/- HH:MM to Server Local Time', WCP_TD) ?></th>
	<td><input type="text" size="10" name="<?php echo $this->prefix; ?>tz_diff" value="<?php echo get_option($this->prefix.'tz_diff', ''); ?>" /></td>
	</tr>
	
	<tr valign="top">
	<th scope="row" style="width:300px"><?php _e('Link to Authors site in footer', WCP_TD) ?></th>
	<td><input type="checkbox" id="<?php echo $this->prefix; ?>promo_link" name="<?php echo $this->prefix; ?>promo_link" value="1" <?php echo get_option($this->prefix.'promo_link', 1) ? ' checked ' : ''; ?> /></td>
	</tr>
	
	</table>
	
	
	<p class="submit">
	<input type="submit" class="button" value="<?php _e('Save Changes', WCP_TD) ?>" />
	</p>
	</form>
	
	</div>
		
<?php		
	}
	
	function help() {
?>
		<h2>Help</h2>
		
		This is still Beta software undergoing development - contact ian.haycox@gmail.com for questions, suggestions etc.


	<h1>World Cup Predictor</h1>

	<p><em>Plugin to manage and present a fantasy football (soccer) competition for the FIFA World Cup 2010</em></p>

	<hr />

	<p>
	<strong>Contributors:</strong>Ian Haycox<br />

	<strong>Donate link:</strong> <a href="http://www.ianhaycox.com/donate">http://www.ianhaycox.com/donate</a><br />
	<strong>Tags:</strong> prediction, fantasy football, competition, soccer, football, fantasy, sidebar, widget, post<br />
	<strong>Requires at least:</strong> 2.8<br />
	<strong>Tested up to:</strong> 2.9.2<br />

	<strong>Stable tag:</strong> 0.4</p>

	<hr />

		<h3>Description</h3>
	<p>This plugin is designed to collect users predictions for each of the matches in the 2010 FIFA World Cup.</p>

<p>Users predict the score for each match in the group and knockout stages and are awarded points based on the accuracy of their prediction.</p>

<p>Match results can also be displayed via additional shortcodes.</p>

<p><strong>Features</strong></p>

<ul>
<li>easy adding of matches, venues, results</li>
<li>entry form for users to make predictions on each World Cup games</li>

<li>sidebar widget to display prediction standings</li>
<li>sidebar widget to display currently logged in users' predictions</li>
<li>countdown timer to next prediction deadline</li>
<li>display of match results and group tables</li>
<li>configurable scoring system to award predictions</li>
</ul>	<hr />

<h3>Installation</h3>
	<p>To install the plugin complete the following steps</p>

<ol>
<li>Unzip the zip-file and upload the content to your Wordpress Plugin directory. Usually <code>/wp-content/plugins</code></li>
<li>Activate the plugin via the Admin plugin page.</li>
<li>Configure Matches etc via the Admin page.</li>
</ol>

	<h3>Upgrade Notice</h3>

	<dl class="error">
		<dt>The database structure has changed between V0.2 and V0.3.  After upgrading to 0.3, ensure you deactivate and reactivate the plugin then re-import the matches.</dt>
		<dd></dd>
	</dl>


<p>For more details on configuration see the Usage section below.</p>	<hr />
		<h3>Frequently Asked Questions</h3>
<ul>
<li>Why are there 64 teams ? - There are extra dummy 'placeholder' teams to fill-in the knockout results tables. Once a team has progressed through the group
stage to the knockout stage edit the match schedule to change the placeholder team to the actual team.</li>
</ul>
<hr />
<h3>Screenshots</h3>
	<ol>
<li><a href="http://s.wordpress.org/extend/plugins/world-cup-predictor/screenshot-1.png">Prediction entry form</a></li>
<li><a href="http://s.wordpress.org/extend/plugins/world-cup-predictor/screenshot-2.png">Group table results display</a></li>
<li><a href="http://s.wordpress.org/extend/plugins/world-cup-predictor/screenshot-3.png">Admin Screens</a></li>
<li><a href="http://s.wordpress.org/extend/plugins/world-cup-predictor/screenshot-4.png">Overview Screen</a></li>
<li><a href="http://s.wordpress.org/extend/plugins/world-cup-predictor/screenshot-5.png">Predictions</a></li>
<li><a href="http://s.wordpress.org/extend/plugins/world-cup-predictor/screenshot-6.png">Knockout tables</a></li>
</ol>	<hr />
		<h3>Changelog</h3>
<p><a href="http://svn.wp-plugins.org/world-cup-predictor/trunk/changelog.txt">ChangeLog</a></p>	<hr />
	
	<h3>Usage</h3>
<p>After installation use Import button on the Admin 'World Cup' overview option to load the teams and match schedules.</p>

<p>Verify the timezone offset on this screen. Any difference should reflect the difference in time between your servers'
local time and South African time. If this setting is incorrect then users may be able to make predictions after kickoff.</p>

<p>The date and times of matches are in South African local time (UTC + 2)</p>

<p>In order to manage the teams, matches etc. the logged in user must have the Worldpress capability <code>wcp2010_manager</code>. By default
the plugin adds this capability to the Administrator and Editor roles. If the World Cup menu option is not available verify that
the currrent user has the <code>wcp2010_manager</code> role. You may need to install a capability manager plugin to manage roles and capabilities.</p>

Users of the blog do not need any special role or capability to make predictions.

<h4>Predictions</h4>

<p>Create a post or page with the shortcode <pre><code>[world-cup-predictor]</code></pre> to display an entry form. Other shortcodes are available to display prediction results
and match results. See below for more details.</p>

<p>Users can only predict on matches that have not yet started. Once a match has started it is removed from the entry form preventing further
predictions.</p>

<h4>Match Results</h4>

<p>As each match is complete, use the Matches admin menu option to enter the goals scored and check the 'Match Finished' checkbox.
The group tables and match results displayed by the shortcodes will be updated.</p>

<p>Once a team is confirmed as coming first or second in the group stage, edit the Matches to change the placeholder team to the
team that has qualified to the next stage.  For example, if South Africa win Group A, edit Match Number 49 and change the
team from 'Winner Group A' to 'South Africa'.</p>

<p>This change is then reflected in the knockout results table.</p>

<h4>Options</h4>

<p>None as yet.</p>

<h4>Points Calculations</h4>

<p>Once a match has finished enter the final score in the Matches tab and check the 'Match Finished' box. The displayed group tables
will then update with the latest team standings.</p>

<p>To update the users' prediction scores, select the match in the 'Predictions' menu and click 'Score Selected'. This updates the
users' scores in the widget and results tables.</p>

<p>Points are assigned according to the settings in the 'Configure Scoring' tab.</p>

<h4>Shortcodes</h4>

<p>You can display an entry form for predictions and the results of matches and predictions with the following shortcodes in a post or page.</p>

<pre><code>[world-cup-predictor]
</code></pre>

<p>Display an entry form for all the configured matches where the kickoff time is before the current date and time.
The form will display a countdown to the next match deadline. Once the match kickoff time has passed the match will
no longer be displayed and users cannot make predictions.</p>

<p>You may optionally supply the attribute <code>stage=n</code> where <code>n</code> is the stage id to limit the matches to a particular group.</p>

<pre><code>[world-cup-predictor tables=1 stage=n, show_results=n]

</code></pre>

<p>Display the group tables with the match results once the match has finished and the results have been entered.
You may optionally supply the attribute <code>stage=n</code> where <code>n</code> is the stage id to limit the matches to a particular group.
You may optionally supply the attribute <code>show_results=n</code> where <code>n</code> = 0 or 1 to hide or show the match results below the group table.</p>

<pre><code>[world-cup-predictor knockout=1]
</code></pre>

<p>Display the results of the knockout stages of the tournament.</p>

<pre><code>[world-cup-predictor results=1 stage=n]
</code></pre>

<p>Display a table of match results. You may optionally supply the attribute <code>stage=n</code> where <code>n</code> is the stage id to limit the matches to a particular group.</p>

<pre><code>[world-cup-predictor scores=n]
</code></pre>

<p>For the currently logged in user display the list of predictions for that user.</p>

<pre><code>`[world-cup-predictor user=1]`
</code></pre>

<p>For each match ID, specified by <code>n</code>, display a table of each users' predictions and the points awarded.</p>

<pre><code>[world-cup-predictor ranking=1 limit=999]
</code></pre>

<p>Display a summary of all the users' points as a ranking table</p>

<h4>User Rankings Widget</h4>

<p>The widget displays similar output to the shortcode <code>[world-cup-predictor ranking=1 limit=999]</code></p>

<p>Drag and drop the widget to a sidebar and configure. Multiple instances of the widget, each configured separately, may be
placed on a sidebar.</p>

<p>If not blank, the URL option adds a link to the full results page using the title below as the link text.</p>

<h4>User Predictions Widget</h4>

<p>The widget displays the list of the currently logged in users' predictions similar in output to the shortcode <code>`[world-cup-predictor user=1]`</code></p>

<p>Drag and drop the widget to a sidebar and configure.</p>


<?php 


	}
	
	
	/**
	 * Set to current JQuery tab
	 * 
	 * @param int $i tab number indexed from 0
	 * @return none
	 */
	function selectTab($i) {
?>
		<script type="text/javascript">
		jQuery(function($) {
		  	$("#<?php echo $this->prefix; ?>tabs").tabs('select', <?php echo $i; ?>);
		});
		</script>
<?php 
	}

	/**
	 * Get a list of registered users in a dropdown select box
	 * 
	 * @param $player_id - Preselect this user
	 */
	function getUsers($user_id, $empty = true, $id = 'user_id', $empty_str = 'All Users') {
		global $wpdb;
		
		$sql = 'SELECT ID,user_login, display_name FROM '.$wpdb->prefix.'users';
		
		$users = $wpdb->get_results( $sql , OBJECT );
		
		$output = '<select name="'.$id.'" id="'.$id.'">';
		if ($empty) $output .= '<option value = "-1">'.$empty_str.'</option>';

		foreach ($users as $row) {
			$output .= "<option ";
			if (!is_null($user_id) && $user_id == $row->ID) {
				$output .= " selected ";
			}
			$output .= "value=\"$row->ID\">$row->user_login ($row->display_name)</option>";
		}
		$output .= "</select>";
		
		return $output;
	}
	
	
	/*
	 * Return the time difference between Server Local time
	 * and South African time. SAST is always UTC+2
	 */
	function getRSADiff() {
		global $wpdb;

		$sql = "SELECT TIMEDIFF(DATE_ADD(UTC_TIMESTAMP(), INTERVAL 2 HOUR), NOW()) AS rsa_diff";
		$result = $wpdb->get_row( $sql , OBJECT );
		return substr($result->rsa_diff, 0, 5);
	}
	
	/**
	 * Gets the current date and time from the server
	 * 
	 * @return MySQL format date/time string YYYY-MM-DD HH:MM:SS
	 */
	function getServerDateTime() {
		global $wpdb;
		
		$sql = 'SELECT NOW() as "now"';
		$result = $wpdb->get_row( $sql , OBJECT );
		return $result->now;
		
	}
	
	/**
	 * Gets the date/time in timezone UTC+2 (i.e. South Africa).
	 * RSA does not operate DST.
	 * 
	 * @return MySQL format date/time string YYYY-MM-DD HH:MM:SS
	 */
	function getRSADateTime() {
		global $wpdb;
		
		$sql = 'SELECT DATE_ADD(UTC_TIMESTAMP(), INTERVAL 2 HOUR) AS rsa_dt';
		$result = $wpdb->get_row( $sql , OBJECT );
		return $result->rsa_dt;
		
	}
	
	/*
	 * Check is [+/-]HH:MM format
	 */
	function is_hhmm($s) {
		return (ereg ("^[+-]{0,1}([0-9]{2}):([0-9]{2})", $s));
	}
	
	/*
	 * Check YYYY-MM-DD HH:MM:SS format
	 */
	function is_datetime($d) {
		return (ereg ("([0-9]{4})-([0-9]{2})-([0-9]{2}) ([0-9]{2}):([0-9]{2}):([0-9]{2})", $d));
	}
	
}