<?php
/**
 * @package WorldCup
 * @version $Id: wcp-predict.class.php 236599 2010-05-02 14:29:16Z ianhaycox $
 * @author ian
 * Copyright Ian Haycox, 2010
 *
 * Prediction forms for the WorldCup plugin.
 *
 */

class WorldCupPredict extends WorldCup {
	
	
	/*
	 * Constructor
	 */
	function __construct() {
		//global $wpdb;
		//$wpdb->show_errors(true); - http://trac.buddypress.org/ticket/2361
		parent::__construct();
	}
	
	/**
	 * Show the prediction form for a logged in user.
	 * 
	 * @param $stage - Show only matches from a particular stage, i.e. Group A  0 == all
	 * @return unknown_type
	 */
	function prediction_form($stage) {
		global $wpdb;
		
		$disabled = '';
		$match = array();
		$output = '';
		$tzdiff = get_option($this->prefix.'tz_diff','00:00');  // Difference between SAST and server time.
		
		$msg_id = $this->prefix . "s$stage";
		
		/*
		 * Protect against remote posts and gets from anonymous users.
		 */
		$logged_in = is_user_logged_in();
		if (!$logged_in) {
			$this->setMessage(__('You must be logged in to predict.', WCP_TD), true);
			$disabled = ' disabled ';
		}
		
		/*
		 * Save entered goals
		 */
		if (isset($_POST[$this->prefix.'save'])) {
			if (isset($_POST['match'])) {
				$match = $_POST['match'];
			}
			$ret = $this->save_prediction($match);
		}
		
		/*
		 * For non logged in users this returns an empty array.
		 * $this->score() default unset match array elements to 0
		 */
		$match = $this->get_predictions();
		
		$stage_filter = '';
		if ($stage) $stage_filter = "AND s.stage_id = $stage";
				
		$sql = "SELECT s.stage_id, match_id, match_no, DATE_FORMAT(kickoff,'%%b %%e %%k:%%i') AS kickoff,
					h.name AS home_team_name, a.name AS away_team_name, is_group,
					home_goals, away_goals, venue_name, stage_name, 
					h.country AS home_country, a.country AS away_country,
					h.team_url AS home_url, a.team_url AS away_url,
					venue_url, stadium, m.wwhen
				FROM 
					{$wpdb->prefix}{$this->prefix}match m,
					{$wpdb->prefix}{$this->prefix}venue v,
					{$wpdb->prefix}{$this->prefix}stage s,
					{$wpdb->prefix}{$this->prefix}team h,
					{$wpdb->prefix}{$this->prefix}team a
				WHERE
					m.venue_id = v.venue_id AND m.stage_id = s.stage_id AND
					m.home_team_id = h.team_id AND m.away_team_id = a.team_id AND
					DATE_ADD(NOW(), INTERVAL %s HOUR_MINUTE) < kickoff
					$stage_filter
				ORDER BY
					sort_order, kickoff";

		$result = $wpdb->get_results( $wpdb->prepare($sql, $tzdiff) , OBJECT );
		
		$save = '<p class="save-button"><input '.$disabled.' class="button" type="submit" value="'.__('Save', WCP_TD).'" name="'.$this->prefix.'save" /></p>';

		$output .= '<div id="'.$msg_id.'">' . $this->printMessage(false) . '</div>' . PHP_EOL;
		
 		$output .= $this->countdown_clock($stage);
		
		$output .= '<form class="'.$this->prefix.'user_form" method="POST">' . PHP_EOL;
		$stage_id = -1;
		foreach ($result as $row) {
			if ($stage_id != $row->stage_id) {
				if ($stage_id != -1) {
					$output .= '</table>' . PHP_EOL;
					$output .= $save . PHP_EOL;
				}
				$output .= '<table class="predictor"><tr><th colspan="4">'.$this->unclean($row->stage_name).'</th></tr>' . PHP_EOL;
				$stage_id = $row->stage_id;
			}
			
			$output .= '<tr><td class="l">'.$this->mklink($this->flag($row->home_country), $row->home_url, $this->unclean($row->home_team_name)) . ' ' . $this->mklink($this->unclean($row->home_team_name), $row->home_url, $this->unclean($row->home_team_name));
			$output .= '</td><td class="cl"><input type="text" value="'.$this->score($match,$row->match_id,'home').'" name="match['.$row->match_id.'][home]" /></td>';
			$output .= '<td class="cr"><input type="text" value="'.$this->score($match,$row->match_id,'away').'" name="match['.$row->match_id.'][away]" /></td><td class="r">';
			$output .= $this->mklink($this->unclean($row->away_team_name), $row->away_url, $this->unclean($row->away_team_name)).' '.$this->mklink($this->flag($row->away_country), $row->away_url, $this->unclean($row->away_team_name)).'</td></tr>' . PHP_EOL;
			
			/*
			 * For knockout stage allow penalty scores.
			 */
			if (!$row->is_group) {
				$output .= '<tr><td class="r">' . __('Penalty', WCP_TD);
				$output .= '</td><td class="cl"><input type="text" value="'.$this->score($match,$row->match_id,'pen_home').'" name="match['.$row->match_id.'][pen_home]" /></td>';
				$output .= '<td class="cr"><input type="text" value="'.$this->score($match,$row->match_id,'pen_away').'" name="match['.$row->match_id.'][pen_away]" /></td><td class="l">';
				$output .= __('shootout', WCP_TD).'</td></tr>' . PHP_EOL;
			}
			$output .= '<tr class="venue"><td  class="sep" colspan="4">'.$this->mklink($row->venue_name,$row->venue_url, $row->stadium).' &ndash; '.$row->kickoff.'</td></tr>' . PHP_EOL;
		}
		if (count($result)) {
			$output .= '</table>' . PHP_EOL;
			$output .= $save . PHP_EOL;
		}
		
		$output .= '<input type="hidden" name="msg_id" value="'.$msg_id.'" />' . PHP_EOL;
		$output .= '</form>' . PHP_EOL;
		
		return $output;
		
	}
	
	/*
	 * Return user entry on form if set
	 */
	function score($match, $match_id, $team) {
		if (is_array($match)) {
			if (isset($match[$match_id])) {
				return $match[$match_id][$team];
			}
		}
		return "0";
	}
	
	/*
	 * Get my predictions
	 */
	function get_predictions() {
		global $wpdb;
		global $current_user;
		
		get_currentuserinfo();
		
		$match = array();
		$sql = "SELECT
					match_id, home_goals, away_goals, home_penalties, away_penalties
				FROM
					{$wpdb->prefix}{$this->prefix}prediction
				WHERE
					user_id = %d";
		$result = $wpdb->get_results($wpdb->prepare($sql, $current_user->ID));
		foreach ($result as $row) {
			$match[$row->match_id] = array('home' => $row->home_goals, 'away' => $row->away_goals, 'pen_home' => $row->home_penalties, 'pen_away' => $row->away_penalties);
		}
		return $match;
	}
	
	/*
	 * Save this submitted prediction
	 */
	function save_prediction($match) {
		global $wpdb;
		global $current_user;
		
		$ret = false;
		if (!is_array($match)) {
			return false;
		}
		
		$this->setMessage(__('Prediction saved', WCP_TD));
		
		/*
		 * Validate
		 */
		foreach ($match as $key=>$m) {
			if (!is_numeric($m['home']) || !is_numeric($m['away'])) {				// TODO check integer (numeric can be 1E23+06)
				$this->setMessage(__('Match score must be numeric', WCP_TD), true);
				return false;
			}
			
			if (isset($m['pen_home'])) {
				if (!is_numeric($m['pen_home'])) {				// TODO check integer (numeric can be 1E23+06)
					$this->setMessage(__('Match score must be numeric', WCP_TD), true);
					return false;
				}
			} else {
				$match[$key]['pen_home'] = 0;
			}
			if (isset($m['pen_away'])) {
				if (!is_numeric($m['pen_away'])) {				// TODO check integer (numeric can be 1E23+06)
					$this->setMessage(__('Match score must be numeric', WCP_TD), true);
					return false;
				}
			} else {
				$match[$key]['pen_away'] = 0;
			}
		}
		
		/*
		 * Save each prediction - checking that they are before the prediction deadline
		 * as the user have have left the form displayed over the deadline (kickoff South African time) !
		 */
		$tzdiff = get_option($this->prefix.'tz_diff','00:00');  // +ve for earlier timezones, -ve for later
																// e.g. PST = 09:00  and Japan = -07:00
																
		get_currentuserinfo();
		foreach ($match as $key=>$m) {
			
			$sql = "SELECT kickoff FROM {$wpdb->prefix}{$this->prefix}match WHERE match_id = %d";
			$match_start = $wpdb->get_row($wpdb->prepare($sql, $key));
			if (!$match_start) {
				$this->setMessage(__('Missing match. Prediction not saved.', WCP_TD), true);
				return false;
			}
			
			$sql = "SELECT prediction_id
					FROM
						{$wpdb->prefix}{$this->prefix}prediction p
					WHERE
						user_id = %d AND match_id = %d";
			$row = $wpdb->get_row($wpdb->prepare($sql, $current_user->ID, $key));
			if ($row) {
				$sql = "UPDATE {$wpdb->prefix}{$this->prefix}prediction
						SET
							home_goals = %d,
							away_goals = %d,
							home_penalties = %d,
							away_penalties = %d,
							wwhen = NOW()
						WHERE
							prediction_id = %d AND DATE_ADD(NOW(), INTERVAL %s HOUR_MINUTE) < %s";
				$ret = $wpdb->query($wpdb->prepare($sql, $m['home'], $m['away'], $m['pen_home'], $m['pen_away'], $row->prediction_id, $tzdiff, $match_start->kickoff));
			} else {
				
				$sql = "INSERT INTO {$wpdb->prefix}{$this->prefix}prediction
							(user_id, match_id, home_goals, away_goals, home_penalties, away_penalties, points)
						SELECT %d, %d, %d, %d, %d, %d, 0 FROM DUAL
						WHERE
							DATE_ADD(NOW(), INTERVAL %s HOUR_MINUTE) < %s";
				$ret = $wpdb->query($wpdb->prepare($sql, $current_user->ID, $key, $m['home'], $m['away'], $m['pen_home'], $m['pen_away'], $tzdiff, $match_start->kickoff));
			}
			if ($ret === false) {  // can return 0 as successfully modified zero rows
				$this->setMessage(__('Error saving prediction', WCP_TD) . $wpdb->last_error, true);
				return false;
			}
			if ($ret == 0) {
				// No rows updated/inserted, so prediction must be too late !!!
				$this->setMessage(__('Some, or all, of your predictions are too late and will not be counted.', WCP_TD), true);
			}
		}
		return $ret;
	}
	
	/*
	 * Save via AJAX
	 */
	function ajax() {
		
		global $current_user;
		
		$match = array();
		get_currentuserinfo();
		
		if (isset($_POST['match'])) {
		
			$match = $_POST['match'];
			$this->save_prediction($match);
		} else {
			$this->setMessage(__('No data to save', WCP_TD), true);
		}
		
		require_once(dirname(__FILE__).'/wcp-reports.class.php');
		$r = new WorldCupReport();
		$ret = array('notice' => $this->printMessage(false), 'preds' => $r->user_predictions());
		
		die(json_encode($ret));
	}
	
	/*
	 * Display a Javascript clock for the next match
	 */
	function countdown_clock($stage) {
		/*
		 * Gets the current date and time from the server and the
		 * date and time of the next prediction deadline.
		 */
		global $wpdb;
		$deadline = array();
		$tzdiff = get_option($this->prefix.'tz_diff','00:00');
		$output = '';
		
		$stage_filter = '';
		if ($stage) $stage_filter = "AND stage_id = $stage";
		
		$sql = "SELECT DATE_FORMAT(DATE_ADD(NOW(), INTERVAL '$tzdiff' HOUR_MINUTE), '%m/%d/%Y %r') AS 'now', 
			 		DATE_FORMAT(MIN(kickoff), '%m/%d/%Y %r') AS 'target'
			 	FROM
					{$wpdb->prefix}{$this->prefix}match
			 	WHERE
			 		kickoff > DATE_ADD(NOW(), INTERVAL '$tzdiff' HOUR_MINUTE) $stage_filter";
		$result = $wpdb->get_row( $sql , OBJECT );
		
		if (is_null($result) || is_null($result->target)) {
			return "";
		}
		
		$deadline['now'] = $result->now;
		$deadline['target'] = $result->target;
		
		/*
		 * If there is a prediction deadline coming up display a Javascript
		 * countdown clock.
		 */
		if ($deadline) {
			
			$output .= '<div class="wcp2010_clock" id="wcp2010_clock_'.$stage.'">Clock</div>';
			$output .= '
		<script type="text/javascript">
			<!--
			var wcp2010_cd'.$stage.' = new wcp2010_countdown(\'wcp2010_cd'.$stage.'\');
			wcp2010_cd'.$stage.'.Div			= "wcp2010_clock_'.$stage.'";
			wcp2010_cd'.$stage.'.TargetDate		= "'.$deadline['target'].'";
			wcp2010_cd'.$stage.'.ServerDate		= "'.$deadline['now'].'";
			wcp2010_cd'.$stage.'.DisplayFormat	= "'.__('Next prediction deadline in', WCP_TD).' %%D%%d, %%H%%h, %%M%%m, %%S%%s";
			wcp2010_cd'.$stage.'.FinishStr    	= "'.__('Too Late - Prediction deadline passed', WCP_TD).'";
			wcp2010_cd'.$stage.'.Setup();
			
			//-->
		</script>
				
			';
		} else {
			$output .= '<div class="wcp2010_clock" id="wcp2010_clock_'.$stage.'">' .
				__('No matches available in this championship.', WCP_TD) . '</div>';
		}
		
		return $output;
	}
	
}


?>