<?php
/**
 * @package WorldCup
 * @version $Id: wcp-reports.class.php 236599 2010-05-02 14:29:16Z ianhaycox $
 * @author ian
 * Copyright Ian Haycox, 2010
 *
 */

/*
 * Match stats
 */
class WorldCupStats {
	var $team_id = 0;
	var $played = 0;
	var $won = 0;
	var $lost = 0;
	var $drawn = 0;
	var $gf = 0;
	var $ga = 0;
	var $points = 0;
	var $stage_id = 0;
	var $team_country = '';
	var $team_name = '';
	var $team_url = '';
	var $sort_order = 0;
}

class WorldCupReport extends WorldCup {

	private $knockout;  // Knockout stage match results
	
	/*
	 * Constructor
	 */
	function __construct() {
		parent::__construct();
	}
	
	
	function table_header($stage_name) {
		
		$team = __('Team',WCP_TD);
		$mp = __('MP', WCP_TD);
		$mp_l = __('Matches Played', WCP_TD);
		$w = __('W', WCP_TD);
		$w_l = __('Won', WCP_TD);
		$d = __('D', WCP_TD);
		$d_l = __('Draw', WCP_TD);
		$l = __('L', WCP_TD);
		$l_l = __('Lost', WCP_TD);
		$gf = __('GF', WCP_TD);
		$gf_l = __('Goals For', WCP_TD);
		$ga = __('GA', WCP_TD);
		$ga_l = __('Goals Against', WCP_TD);
		$p = __('Pts', WCP_TD);
		$p_l = __('Points', WCP_TD);
		
		$output = <<<EOT
		
		<table class="group zebra">
		<tr><th colspan="8"><h3>$stage_name</h3></th></tr>
		<tr class="row-header">
			<th class="col1">$team</th>
			<th class="col-goal"><span title="$mp_l">$mp</span></th>
			<th class="col-goal"><span title="$w_l">$w</span></th>
			<th class="col-goal"><span title="$d_l">$d</span></th>
			<th class="col-goal"><span title="$l_l">$l</span></th>
			<th class="col-goal"><span title="$gf_l">$gf</span></th>
			<th class="col-goal"><span title="$ga_l">$ga</span></th>
			<th class="col-goal"><span title="$p_l">$p</span></th>
		</tr>
EOT;
		return $output;
	}
	
	function table_row($row) {
		
		$flag = $this->mklink($this->flag($row->team_country), $this->unclean($row->team_url), $this->unclean($row->team_name));
		$name = $this->mklink($this->unclean($row->team_name), $this->unclean($row->team_url), $this->unclean($row->team_name));
		
		$output = <<<EOT
			<tr class="row">
				<th class="col-team">$flag $name</th>
				<td class="col-goal">$row->played</td>
				<td class="col-goal">$row->won</td>
				<td class="col-goal">$row->drawn</td>
				<td class="col-goal">$row->lost</td>
				<td class="col-goal">$row->gf</td>
				<td class="col-goal">$row->ga</td>
				<td class="col-goal">$row->points</td>
			</tr>
EOT;
		return $output;
	}
	
	/*
	 * For each team - calculate the matches played, points, etc.
	 */
	function calculate_group_tables() {
		global $wpdb;
		
		/*
		 * All matches
		 */
		$sql = "SELECT DISTINCT t.team_id, t.name AS team_name, t.team_url, t.country AS team_country, stage_name, s.stage_id, s.sort_order
				FROM
					{$wpdb->prefix}{$this->prefix}stage s,
					{$wpdb->prefix}{$this->prefix}team t,
					{$wpdb->prefix}{$this->prefix}match m
				WHERE
					(m.home_team_id = t.team_id OR m.away_team_id = t.team_id) AND
					m.stage_id = s.stage_id AND s.is_group = 1 
				ORDER BY
					s.sort_order";
		$teams = $wpdb->get_results($wpdb->prepare($sql), OBJECT_K);
		
		/*
		 * Matches played
		 */
		$sql = "SELECT t.team_id, COUNT(*) AS played, t.name, s.stage_id FROM
					{$wpdb->prefix}{$this->prefix}stage s,
					{$wpdb->prefix}{$this->prefix}team t,
					{$wpdb->prefix}{$this->prefix}match m
				WHERE
					(m.home_team_id = t.team_id OR m.away_team_id = t.team_id) AND
					m.stage_id = s.stage_id AND s.is_group = 1 AND m.is_result = 1 
				GROUP BY
					t.team_id";
		$mp =  @$wpdb->get_results($wpdb->prepare($sql), OBJECT_K);
		
		/*
		 * Matches won
		 */
		$sql = "SELECT t.team_id, COUNT(*) AS won, t.name FROM
					{$wpdb->prefix}{$this->prefix}stage s,
					{$wpdb->prefix}{$this->prefix}team t,
					{$wpdb->prefix}{$this->prefix}match m
				WHERE
					((m.home_team_id = t.team_id AND m.home_goals > m.away_goals) OR
					 (m.away_team_id = t.team_id AND m.away_goals > m.home_goals))
					AND m.stage_id = s.stage_id AND s.is_group = 1 AND m.is_result = 1 
				GROUP BY
					t.team_id";
		$won =  @$wpdb->get_results($wpdb->prepare($sql), OBJECT_K);
					
		/*
		 * Matches lost
		 */
		$sql = "SELECT t.team_id, COUNT(*) AS lost, t.name FROM
					{$wpdb->prefix}{$this->prefix}stage s,
					{$wpdb->prefix}{$this->prefix}team t,
					{$wpdb->prefix}{$this->prefix}match m
				WHERE
					((m.home_team_id = t.team_id AND m.home_goals < m.away_goals) OR
					 (m.away_team_id = t.team_id AND m.away_goals < m.home_goals))
					AND m.stage_id = s.stage_id AND s.is_group = 1 AND m.is_result = 1 
				GROUP BY
					t.team_id";
		$lost =  @$wpdb->get_results($wpdb->prepare($sql), OBJECT_K);
		
		/*
		 * Matches drawn
		 */
		$sql = "SELECT t.team_id, COUNT(*) AS drawn, t.name FROM
					{$wpdb->prefix}{$this->prefix}stage s,
					{$wpdb->prefix}{$this->prefix}team t,
					{$wpdb->prefix}{$this->prefix}match m
				WHERE
					((m.home_team_id = t.team_id AND m.home_goals = m.away_goals) OR
					 (m.away_team_id = t.team_id AND m.home_goals = m.away_goals))
					AND m.stage_id = s.stage_id AND s.is_group = 1 AND m.is_result = 1 
				GROUP BY
					t.team_id";
		$drawn =  @$wpdb->get_results($wpdb->prepare($sql), OBJECT_K);
		
		/*
		 * Goals for and against
		 */
		$sql = "SELECT home_team_id AS team_id, SUM(home_goals) AS gf, SUM(away_goals) AS ga FROM
					{$wpdb->prefix}{$this->prefix}stage s,
					{$wpdb->prefix}{$this->prefix}match m
				WHERE
					m.stage_id = s.stage_id AND s.is_group = 1 AND m.is_result = 1 
				GROUP BY
					m.home_team_id";
		$goals1 =  @$wpdb->get_results($wpdb->prepare($sql), OBJECT_K);
		$sql = "SELECT away_team_id AS team_id, SUM(away_goals) AS gf, SUM(home_goals) AS ga FROM
					{$wpdb->prefix}{$this->prefix}stage s,
					{$wpdb->prefix}{$this->prefix}match m
				WHERE
					m.stage_id = s.stage_id AND s.is_group = 1 AND m.is_result = 1 
				GROUP BY
					m.away_team_id";
		$goals2 =  @$wpdb->get_results($wpdb->prepare($sql), OBJECT_K);
		
		$stats = array();
		
		/*
		 * Merge the results from above.
		 */
		foreach ($teams as $team_id=>$val) {
			
			$temp = new WorldCupStats();
			
			$temp->team_id = $team_id;
			$temp->stage_id = $val->stage_id;
			$temp->team_url = $val->team_url;
			$temp->stage_name = $val->stage_name;
			$temp->team_country = $val->team_country;
			$temp->team_name = $val->team_name;
			$temp->sort_order = $val->sort_order;
			
			if (is_array($mp) && array_key_exists($team_id,$mp)) {
				$temp->played = $mp[$team_id]->played;
			}
			if (is_array($won) && array_key_exists($team_id,$won)) {
				$temp->won = $won[$team_id]->won;
				$temp->points += (3 * $temp->won);
			}
			if (is_array($lost) && array_key_exists($team_id,$lost)) {
				$temp->lost = $lost[$team_id]->lost;
			}
			if (is_array($drawn) && array_key_exists($team_id,$drawn)) {
				$temp->drawn = $drawn[$team_id]->drawn;
				$temp->points += (1 * $temp->drawn);
			}
			if (is_array($goals1) && array_key_exists($team_id,$goals1)) {
				$temp->gf = $goals1[$team_id]->gf;
				$temp->ga = $goals1[$team_id]->ga;
			}
			if (is_array($goals2) && array_key_exists($team_id,$goals2)) {
				$temp->gf += $goals2[$team_id]->gf;
				$temp->ga += $goals2[$team_id]->ga;
			}
			
			$stats[] = $temp;
		}
		
		usort($stats, array($this, 'stats_sort'));
		
		update_option($this->prefix.'group_stats', $stats);  // Cache stats
		
		return $stats;
	}
	
	/*
	 * User sort
	 * 
	 * Sort by group sort order (keep groups together), then points, then matches played, then,
	 * 
	 * From http://en.wikipedia.org/wiki/2010_FIFA_World_Cup#Group_stage
	 * 
	 * Tie-breaking criteria
	 * 
	 * For the World Cup tournament, FIFA uses the following criteria to rank teams in the Group Stage.[88]
	 * 
	 * 1. greatest number of points in all group matches;
	 * 2. goal difference in all group matches;
	 * 3. greatest number of goals scored in all group matches.
	 * 4. greatest number of points in matches between tied teams;
	 * 5. goal difference in matches between tied teams;
	 * 6. greatest number of goals scored in matches between tied teams;
	 * 7. drawing of lots by the FIFA Organising Committee.
	 * 
	 */
	function stats_sort($a, $b) {
		
		if ($a->sort_order == $b->sort_order) {
			if ($a->points == $b->points) {					// Tie break 1.
				if ($a->played == $b->played) {				// Before all matches played promote better
															// points for lesser matches
					$agd = $a->gf - $a->ga;  // Goal difference
					$bgd = $b->gf - $b->ga;
					if ($agd == $bgd) {						// Tie break 2
						
						if ($a->gf == $b->gf) {				// Tie break 3
							return 0;						// TODO - Tie break 4,5,6,7
						}
		    			return ($a->gf < $b->gf) ? 1 : -1;		
					}
		    		return ($agd < $bgd) ? 1 : -1;		
			    }
				return ($a->played < $b->played) ? -1 : 1;
			}
		    return ($a->points < $b->points) ? 1 : -1;
		}
	    return ($a->sort_order < $b->sort_order) ? -1 : 1;		
	}
	
	/*
	 * Return the cached group tables
	 */
	function group_match_stats() {
		$stats = get_option($this->prefix.'group_stats');  // Get cached stats
		if (!$stats) {
			$stats = $this->calculate_group_tables();
		}
		return $stats;
	}
	
	/*
	 * Display group tables.
	 * 
	 * $stage 0 == all, or limit to stage_id
	 * $show_results - if true display match results under group table.
	 */
	function group_tables($stage, $show_results, $width='100%') {
		
		$output = '';
		
		$stats = $this->group_match_stats();
		
		$stage_id = -1;
		foreach ($stats as $row) {
			if ($stage && $stage != $row->stage_id) continue;
			if ($stage_id != $row->stage_id) {
				if ($stage_id != -1) {
					$output .= '</table>' . PHP_EOL;
					if ($show_results) $output .= $this->results($stage_id, $width);
				}
				$output .= $this->table_header($this->unclean($row->stage_name));
				$stage_id = $row->stage_id;
			}
			$output .= $this->table_row($row);
		}
		if (count($stats) && !empty($output)) {
			$output .= '</table>' . PHP_EOL;
			if ($show_results) $output .= $this->results($stage_id, $width);
		}
		
		return $output;
	}
	
	
	/*
	 * Display team name - bold if winner, append (a.e.t) or (pen.) if required.
	 */
	function team_name($row, $home, $justify = true) {
		$output = '';
		$suffix = '';
		$winner1 = '';
		$winner2 = '';
		
		if ($home) {
			$flag = $this->mklink($this->flag($row->home_country), $row->home_url, $this->unclean($row->home_team_name));
			$team = $this->mklink($this->unclean($row->home_team_name), $row->home_url, $this->unclean($row->home_team_name));
			if (!$row->is_group && $row->is_result) {
				
				// Bold winner
				if ($row->home_goals + $row->home_penalties > $row->away_goals + $row->away_penalties) {
					$winner1 = '<strong>';
					$winner2 = '</strong>';
					if ($row->home_penalties != 0 || $row->away_penalties != 0) {
						$suffix = ' (<span title="'.__('Penalties', WCP_TD).'">' . __('pen.', WCP_TD) . '</span>)';
					} elseif ($row->extra_time) {
						$suffix = ' (<span title="'.__('Extra Time', WCP_TD).'">' . __('a.e.t.', WCP_TD) . '</span>)';
					}
				}
			}
			$output = "$flag $winner1{$team}{$winner2} $suffix";
			
		} else {
			$flag = $this->mklink($this->flag($row->away_country), $row->away_url, $this->unclean($row->away_team_name));
			$team = $this->mklink($this->unclean($row->away_team_name), $row->away_url, $this->unclean($row->away_team_name));
			if (!$row->is_group && $row->is_result) {
				
				// Bold winner
				if ($row->away_goals + $row->away_penalties > $row->home_goals + $row->home_penalties) {
					$winner1 = '<strong>';
					$winner2 = '</strong>';
					if ($row->home_penalties != 0 || $row->away_penalties != 0) {
						$suffix = ' (<span title="'.__('Penalties', WCP_TD).'">' . __('pen.', WCP_TD) . '</span>)';
					} elseif ($row->extra_time) {
						$suffix = ' (<span title="'.__('Extra Time', WCP_TD).'">' . __('a.e.t.', WCP_TD) . '</span>)';
					}
				}
			}
			if ($justify) {
				$output = "$suffix $winner1{$team}{$winner2} $flag";
			} else {
				$output = "$flag $winner1{$team}{$winner2} $suffix";
			}
		}
		
		return $output;
	}
	
	/*
	 * Display teams score - bold if winner - optionally add penalties (n)
	 */
	function team_score($row, $home) {
		$output = '';
		$pen = '';
		$winner1 = '';
		$winner2 = '';
		
		if ($home) {
			if (!$row->is_group && $row->is_result) {
				
				// Bold winner
				if ($row->home_goals + $row->home_penalties > $row->away_goals + $row->away_penalties) {
					$winner1 = '<strong>';
					$winner2 = '</strong>';
				}
				if ($row->home_penalties != 0 || $row->away_penalties != 0) {
					$pen = ' ('.$row->home_penalties.')';
				}
			}
			$output = "{$winner1}{$row->home_goals}{$pen}{$winner2}";
			
		} else {
			if (!$row->is_group && $row->is_result) {
				
				// Bold winner
				if ($row->away_goals + $row->away_penalties > $row->home_goals + $row->home_penalties) {
					$winner1 = '<strong>';
					$winner2 = '</strong>';
				}
				if ($row->home_penalties != 0 || $row->away_penalties != 0) {
					$pen = ' ('.$row->away_penalties.')';
				}
			}
			$output = "{$winner1}{$row->away_goals}{$pen}{$winner2}";
		}
		
		return $output;
	}
	
	/*
	 * Match results in a simple table for each stage
	 */
	function results($stage, $width="100%") {
		global $wpdb;
		
		$output = '';
		$stage_filter = '';
		if ($stage) $stage_filter = "AND s.stage_id = $stage";
				
		$sql = "SELECT s.stage_id, match_id, match_no, DATE_FORMAT(kickoff,'%%b %%e %%k:%%i') AS kickoff,
					h.name AS home_team_name, a.name AS away_team_name,
					home_goals, away_goals, home_penalties, away_penalties, venue_name, stage_name, 
					h.country AS home_country, a.country AS away_country,
					h.team_url AS home_url, a.team_url AS away_url,
					venue_url, stadium, m.wwhen, is_group, is_result, extra_time
				FROM 
					{$wpdb->prefix}{$this->prefix}match m,
					{$wpdb->prefix}{$this->prefix}venue v,
					{$wpdb->prefix}{$this->prefix}stage s,
					{$wpdb->prefix}{$this->prefix}team h,
					{$wpdb->prefix}{$this->prefix}team a
				WHERE
					m.venue_id = v.venue_id AND m.stage_id = s.stage_id AND
					m.home_team_id = h.team_id AND m.away_team_id = a.team_id AND
					m.is_result = 1
					$stage_filter
				ORDER BY
					sort_order, kickoff";

		$result = $wpdb->get_results( $wpdb->prepare($sql) , OBJECT );
		
		if ($result) $output .= '<table style="table-layout:fixed;margin-top:1em;width:'.$width.'" class="predictor">' .PHP_EOL;
		
		foreach ($result as $row) {
			$output .= '<tr><td class="l">'.$this->team_name($row, true);
			$output .= '</td><td class="cl">'.$this->team_score($row, true).'</td>';
			$output .= '<td class="c">&ndash;</td>';
			$output .= '<td class="cr">'.$this->team_score($row, false).'</td><td class="r">';
			$output .= $this->team_name($row, false).'</td></tr>' . PHP_EOL;
			$output .= '<tr class="venue"><td  class="sep" colspan="5">'.$this->mklink($row->venue_name,$row->venue_url, $row->stadium).' &ndash; '.$row->kickoff.'</td></tr>' . PHP_EOL;
		}				
		
		if ($result) $output .= "</table>" .PHP_EOL;
		
		return $output;
	}
	
	/*
	 * Get match details
	 */
	function get_stage2_matches() {
		global $wpdb;
		
		$sql = "SELECT match_no, s.stage_id, match_id, DATE_FORMAT(kickoff,'%%b %%e') AS kickoff,
					h.name AS home_team_name, a.name AS away_team_name,
					home_goals, away_goals, home_penalties, away_penalties, venue_name, stage_name, 
					h.country AS home_country, a.country AS away_country,
					h.team_url AS home_url, a.team_url AS away_url,
					venue_url, stadium, is_result, is_group, extra_time
				FROM 
					{$wpdb->prefix}{$this->prefix}match m,
					{$wpdb->prefix}{$this->prefix}venue v,
					{$wpdb->prefix}{$this->prefix}stage s,
					{$wpdb->prefix}{$this->prefix}team h,
					{$wpdb->prefix}{$this->prefix}team a
				WHERE
					m.venue_id = v.venue_id AND m.stage_id = s.stage_id AND
					m.home_team_id = h.team_id AND m.away_team_id = a.team_id AND
					match_no > 48";
		
		return $wpdb->get_results($wpdb->prepare($sql), OBJECT_K);
	}
	
	function stage($i) {
		return "<!--S{$i}-->" . $this->clean($this->knockout[$i]->stage_name);
	}
	
	function venue($i) {
		
		$url = $this->mklink($this->clean($this->knockout[$i]->venue_name),
								$this->clean($this->knockout[$i]->venue_url),
								$this->clean($this->knockout[$i]->stadium));
		
		return "<!--V{$i}-->{$this->knockout[$i]->kickoff} &ndash; $url";
	}
	
	function home_score($i) {
		return "<!--GH{$i}-->" . $this->team_score($this->knockout[$i], true);
	}
	
	function home_team($i) {
		return "<!--TH{$i}-->" . $this->team_name($this->knockout[$i], true);
	}
	
	function away_score($i) {
		return "<!--GA{$i}-->" . $this->team_score($this->knockout[$i], false);
	}
	
	function away_team($i) {
		return "<!--TA{$i}-->" . $this->team_name($this->knockout[$i], false, false);
	}
	
	/*
	 * Display the knockout match tables
	 */
	function knockout() {
		
		$flag = array('xxx' => '');
		
		$this->knockout = $this->get_stage2_matches();
		
		$output = <<<EOT
<table cellspacing="0" cellpadding="0" border="0" style="margin: 1em 2em 1em 1em; font-size: 85%;">
<tbody><tr>
<td height="5"></td>
<td bgcolor="#f2f2f2" align="center" style="border: 1px solid rgb(170, 170, 170);" colspan="2">{$this->stage(49)}</td>
<td colspan="2"></td>
<td bgcolor="#f2f2f2" align="center" style="border: 1px solid rgb(170, 170, 170);" colspan="2">{$this->stage(57)}</td>
<td colspan="2"></td>
<td bgcolor="#f2f2f2" align="center" style="border: 1px solid rgb(170, 170, 170);" colspan="2">{$this->stage(61)}</td>
<td colspan="2"></td>
<td bgcolor="#f2f2f2" align="center" style="border: 1px solid rgb(170, 170, 170);" colspan="2">{$this->stage(64)}</td>
</tr>
<tr>
<td height="5"></td>
<td width="150">&nbsp;</td>
<td width="40">&nbsp;</td>
<td width="15">&nbsp;</td>
<td width="20">&nbsp;</td>
<td width="150">&nbsp;</td>
<td width="40">&nbsp;</td>
<td width="15">&nbsp;</td>
<td width="20">&nbsp;</td>
<td width="150">&nbsp;</td>
<td width="40">&nbsp;</td>
<td width="15">&nbsp;</td>
<td width="20">&nbsp;</td>
<td width="150">&nbsp;</td>
<td width="40">&nbsp;</td>
</tr>
<tr>
<td height="5"></td>
<td rowspan="2" colspan="2">{$this->venue(49)}</td>
<td style="border-style: solid; border-color: black; border-width: 0pt 0pt 1px;" rowspan="4">&nbsp;</td>
<td style="border-style: solid; border-color: black; border-width: 0pt 0pt 1px;" rowspan="7">&nbsp;</td>
<td rowspan="3" colspan="2"></td>
<td style="border-style: solid; border-color: black; border-width: 0pt 0pt 1px;" rowspan="7">&nbsp;</td>
<td style="border-style: solid; border-color: black; border-width: 0pt 0pt 1px;" rowspan="13">&nbsp;</td>
<td rowspan="9" colspan="2"></td>
<td style="border-style: solid; border-color: black; border-width: 0pt 0pt 1px;" rowspan="13">&nbsp;</td>
<td style="border-style: solid; border-color: black; border-width: 0pt 0pt 1px;" rowspan="25">&nbsp;</td>
<td rowspan="21" colspan="2"></td>
</tr>
<tr>
<td height="5"></td>
</tr>
<tr>
<td height="5"></td>
<td bgcolor="#f9f9f9" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->home_team(49)}</td>
<td bgcolor="#f9f9f9" align="center" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->home_score(49)}</td>
</tr>
<tr>
<td height="5"></td>
<td rowspan="2" colspan="2">{$this->venue(58)}</td>
</tr>
<tr>
<td height="5"></td>
<td bgcolor="#f9f9f9" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->away_team(49)}</td>
<td bgcolor="#f9f9f9" align="center" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->away_score(49)}</td>
<td style="border-style: solid; border-color: black; border-width: 2px 3px 1px 0pt;" rowspan="6">&nbsp;</td>
</tr>
<tr>
<td height="5"></td>
<td bgcolor="#f9f9f9" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->home_team(58)}</td>
<td bgcolor="#f9f9f9" align="center" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->home_score(58)}</b></td>
</tr>
<tr>
<td height="5"></td>
<td rowspan="2" colspan="2">{$this->venue(50)}</td>
</tr>
<tr>
<td height="5"></td>
<td style="border-style: solid; border-color: black; border-width: 2px 0pt 1px;" rowspan="12">&nbsp;</td>
<td bgcolor="#f9f9f9" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->away_team(58)}</td>
<td bgcolor="#f9f9f9" align="center" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->away_score(58)}</td>
<td style="border-style: solid; border-color: black; border-width: 2px 3px 1px 0pt;" rowspan="12">&nbsp;</td>
</tr>
<tr>
<td height="5"></td>
<td bgcolor="#f9f9f9" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->home_team(50)}</td>
<td bgcolor="#f9f9f9" align="center" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->home_score(50)}</td>
</tr>
<tr>
<td height="5"></td>
<td rowspan="6" colspan="2"></td>
<td rowspan="2" colspan="2">{$this->venue(61)}</td>
</tr>
<tr>
<td height="5"></td>
<td bgcolor="#f9f9f9" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->away_team(50)}</td>
<td bgcolor="#f9f9f9" align="center" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->away_score(50)}</td>
<td style="border-style: solid; border-color: black; border-width: 2px 0pt 1px;" rowspan="6">&nbsp;</td>
</tr>
<tr>
<td height="5"></td>
<td bgcolor="#f9f9f9" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->home_team(61)}</td>
<td bgcolor="#f9f9f9" align="center" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->home_score(61)}</td>
</tr>
<tr>
<td height="5"></td>
<td rowspan="2" colspan="2">{$this->venue(53)}</td>
</tr>
<tr>
<td height="5"></td>
<td style="border-style: solid; border-color: black; border-width: 2px 0pt 1px;" rowspan="24">&nbsp;</td>
<td bgcolor="#f9f9f9" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->away_team(61)}</td>
<td bgcolor="#f9f9f9" align="center" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->away_score(61)}</td>
<td style="border-style: solid; border-color: black; border-width: 2px 3px 1px 0pt;" rowspan="24">&nbsp;</td>
</tr>
<tr>
<td height="5"></td>
<td bgcolor="#f9f9f9" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->home_team(53)}</td>
<td bgcolor="#f9f9f9" align="center" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->home_score(53)}</td>
</tr>
<tr>
<td height="5"></td>
<td rowspan="2" colspan="2">{$this->venue(57)}</td>
<td rowspan="18" colspan="2"></td>
</tr>
<tr>
<td height="5"></td>
<td bgcolor="#f9f9f9" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">&nbsp;{$this->away_team(53)}</td>
<td bgcolor="#f9f9f9" align="center" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">&nbsp;{$this->away_score(53)}</td>
<td style="border-style: solid; border-color: black; border-width: 2px 3px 1px 0pt;" rowspan="6">&nbsp;</td>
</tr>
<tr>
<td height="5"></td>
<td bgcolor="#f9f9f9" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->home_team(57)}</td>
<td bgcolor="#f9f9f9" align="center" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->home_score(57)}</td>
</tr>
<tr>
<td height="5"></td>
<td rowspan="2" colspan="2">{$this->venue(54)}</td>
</tr>
<tr>
<td height="5"></td>
<td style="border-style: solid; border-color: black; border-width: 2px 0pt 1px;" rowspan="12">&nbsp;</td>
<td bgcolor="#f9f9f9" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->away_team(57)}</td>
<td bgcolor="#f9f9f9" align="center" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->away_score(57)}</td>
<td style="border-style: solid; border-color: black; border-width: 2px 0pt 1px;" rowspan="12">&nbsp;</td>
</tr>
<tr>
<td height="5"></td>
<td bgcolor="#f9f9f9" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->home_team(54)}</td>
<td bgcolor="#f9f9f9" align="center" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">&nbsp;{$this->home_score(54)}</td>
</tr>
<tr>
<td height="5"></td>
<td rowspan="6" colspan="2"></td>
<td rowspan="2" colspan="2">{$this->venue(64)}</td>
</tr>
<tr>
<td height="5"></td>
<td bgcolor="#f9f9f9" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->away_team(54)}</td>
<td bgcolor="#f9f9f9" align="center" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->away_score(54)}</td>
<td style="border-style: solid; border-color: black; border-width: 2px 0pt 1px;" rowspan="6">&nbsp;</td>
</tr>
<tr>
<td height="5"></td>
<td bgcolor="#f9f9f9" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->home_team(64)}</td>
<td bgcolor="#f9f9f9" align="center" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->home_score(64)}</td>
</tr>
<tr>
<td height="5"></td>
<td rowspan="2" colspan="2">{$this->venue(52)}</td>
</tr>
<tr>
<td height="5"></td>
<td style="border-style: solid; border-color: black; border-width: 2px 0pt 0pt;" rowspan="23">&nbsp;</td>
<td bgcolor="#f9f9f9" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->away_team(64)}</td>
<td bgcolor="#f9f9f9" align="center" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->away_score(64)}</td>
</tr>
<tr>
<td height="5"></td>
<td bgcolor="#f9f9f9" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->home_team(52)}</td>
<td bgcolor="#f9f9f9" align="center" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->home_score(52)}</td>
</tr>
<tr>
<td height="5"></td>
<td rowspan="2" colspan="2">{$this->venue(59)}</td>
<td rowspan="10" colspan="2"></td>
</tr>
<tr>
<td height="5"></td>
<td bgcolor="#f9f9f9" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->away_team(52)}</td>
<td bgcolor="#f9f9f9" align="center" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->away_score(52)}</td>
<td style="border-style: solid; border-color: black; border-width: 2px 3px 1px 0pt;" rowspan="6">&nbsp;</td>
</tr>
<tr>
<td height="5"></td>
<td bgcolor="#f9f9f9" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->home_team(59)}</td>
<td bgcolor="#f9f9f9" align="center" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->home_score(59)}</td>
</tr>
<tr>
<td height="5"></td>
<td rowspan="2" colspan="2">{$this->venue(51)}</td>
</tr>
<tr>
<td height="5"></td>
<td style="border-style: solid; border-color: black; border-width: 2px 0pt 1px;" rowspan="12">&nbsp;</td>
<td bgcolor="#f9f9f9" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->away_team(59)}</td>
<td bgcolor="#f9f9f9" align="center" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->away_score(59)}</td>
<td style="border-style: solid; border-color: black; border-width: 2px 3px 1px 0pt;" rowspan="12">&nbsp;</td>
</tr>
<tr>
<td height="5"></td>
<td bgcolor="#f9f9f9" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->home_team(51)}</td>
<td bgcolor="#f9f9f9" align="center" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->home_score(51)}</td>
</tr>
<tr>
<td height="5"></td>
<td rowspan="6" colspan="2"></td>
<td rowspan="2" colspan="2">{$this->venue(62)}</td>
</tr>
<tr>
<td height="5"></td>
<td bgcolor="#f9f9f9" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->away_team(51)}</td>
<td bgcolor="#f9f9f9" align="center" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->away_score(51)}</td>
<td style="border-style: solid; border-color: black; border-width: 2px 0pt 1px;" rowspan="6">&nbsp;</td>
</tr>
<tr>
<td height="5"></td>
<td bgcolor="#f9f9f9" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->away_team(62)}</td>
<td bgcolor="#f9f9f9" align="center" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->away_score(62)}</td>
</tr>
<tr>
<td height="5"></td>
<td rowspan="2" colspan="2">{$this->venue(55)}</td>
</tr>
<tr>
<td height="5"></td>
<td style="border-style: solid; border-color: black; border-width: 2px 0pt 0pt;" rowspan="11">&nbsp;</td>
<td bgcolor="#f9f9f9" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->home_team(62)}</td>
<td bgcolor="#f9f9f9" align="center" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->home_score(62)}</td>
<td style="border-style: solid; border-color: black; border-width: 2px 0pt 0pt;" rowspan="11">&nbsp;</td>
<td bgcolor="#f2f2f2" align="center" style="border: 1px solid rgb(170, 170, 170);" rowspan="2" colspan="2">{$this->stage(63)}</td>
</tr>
<tr>
<td height="5"></td>
<td bgcolor="#f9f9f9" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->home_team(55)}</td>
<td bgcolor="#f9f9f9" align="center" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->home_score(55)}</td>
</tr>
<tr>
<td height="5"></td>
<td rowspan="2" colspan="2">{$this->venue(60)}</td>
<td rowspan="9" colspan="2"></td>
<td rowspan="2" colspan="2">{$this->venue(63)}</td>
</tr>
<tr>
<td height="5"></td>
<td bgcolor="#f9f9f9" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->away_team(55)}</td>
<td bgcolor="#f9f9f9" align="center" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->away_score(55)}</td>
<td style="border-style: solid; border-color: black; border-width: 2px 3px 1px 0pt;" rowspan="6">&nbsp;</td>
</tr>
<tr>
<td height="5"></td>
<td bgcolor="#f9f9f9" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->home_team(60)}</td>
<td bgcolor="#f9f9f9" align="center" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->home_score(60)}</td>
<td bgcolor="#f9f9f9" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->home_team(63)}</td>
<td bgcolor="#f9f9f9" align="center" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->home_score(63)}</td>
</tr>
<tr>
<td height="5"></td>
<td rowspan="2" colspan="2">{$this->venue(56)}</td>
</tr>
<tr>
<td height="5"></td>
<td style="border-style: solid; border-color: black; border-width: 2px 0pt 0pt;" rowspan="5">&nbsp;</td>
<td bgcolor="#f9f9f9" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->away_team(60)}</td>
<td bgcolor="#f9f9f9" align="center" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->away_score(60)}</td>
<td style="border-style: solid; border-color: black; border-width: 2px 0pt 0pt;" rowspan="5">&nbsp;</td>
<td bgcolor="#f9f9f9" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->away_team(63)}</td>
<td bgcolor="#f9f9f9" align="center" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->away_score(63)}</td>
</tr>
<tr>
<td height="5"></td>
<td bgcolor="#f9f9f9" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->home_team(56)}</td>
<td bgcolor="#f9f9f9" align="center" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->home_score(56)}</td>
</tr>
<tr>
<td height="5"></td>
<td rowspan="3" colspan="2"></td>
<td rowspan="3" colspan="2"></td>
</tr>
<tr>
<td height="5"></td>
<td bgcolor="#f9f9f9" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->away_team(56)}</td>
<td bgcolor="#f9f9f9" align="center" style="border: 1px solid rgb(170, 170, 170);" rowspan="2">{$this->away_score(56)}</td>
<td style="border-style: solid; border-color: black; border-width: 2px 0pt 0pt;" rowspan="2">&nbsp;</td>
</tr>
<tr>
<td height="5"></td>
</tr>
</tbody></table>

EOT;
		return $output;
	}
	
	/**
	 * Display a ranking table of all predictions for all matches
	 * 
	 * @param unknown_type $limit
	 * @param unknown_type $avatar
	 * @return unknown_type
	 */
	function user_ranking($limit = 999999, $avatar = false) {
		global $wpdb;
		
		$output = '';
		
		if (!is_numeric($limit)) {
			$limit = 999999;
		}
		
		$sql = "SELECT SUM(points) AS total, u.display_name, u.ID
				FROM
					{$wpdb->prefix}{$this->prefix}prediction p,
					{$wpdb->prefix}{$this->prefix}match m,
					{$wpdb->prefix}users u
				WHERE
					p.user_id = u.ID AND m.is_result = 1 AND p.match_id = m.match_id
				GROUP BY
					user_id
				ORDER BY
					total DESC
				LIMIT %d";
					
		$result = $wpdb->get_results($wpdb->prepare($sql, $limit));
		$output .= "<table class='group zebra'><tr class='row-header'><th class='col1'>User</th><th class='col-goal'>Points</th></tr>" . PHP_EOL;
		foreach ($result as $row) {
			$output .= "<tr class='row'><td class='col-user'>".($avatar ? get_avatar( 1, '16', '', $row->display_name ) : '').' '.$row->display_name."</td><td class='col-goal'>$row->total</td></tr>" . PHP_EOL;
		}
		$output .= "</table>" . PHP_EOL;
		
		return $output;
	}
	
	/**
	 * Display the guesses for each match and the points awarded
	 * 
	 * @param $match_id
	 * @param $limit
	 */
	function user_scores($match_id, $limit = 999999, $user_id = -1) {
		global $wpdb;
		
		$output = '';
		$avatar = false;
		
		if (!is_numeric($limit)) {
			$limit = 999999;
		}
		
		$filter_user = '';
		if ($user_id != -1) $filter_user = " AND u.ID = $user_id ";
		
		$sql = "SELECT
					h.country AS home_country, a.country AS away_country,
					h.team_url AS home_url, a.team_url AS away_url,
					h.name AS home_team_name, home_goals, home_penalties,
					a.name AS away_team_name, away_goals, away_penalties,
					is_group, extra_time, is_result
				FROM
					{$wpdb->prefix}{$this->prefix}match m,
					{$wpdb->prefix}{$this->prefix}stage s,
					{$wpdb->prefix}{$this->prefix}team h,
					{$wpdb->prefix}{$this->prefix}team a
				WHERE
					s.stage_id = m.stage_id AND m.match_id = %d AND m.is_result = 1 AND
					h.team_id = m.home_team_id AND a.team_id = m.away_team_id";
					
		$toprow = $wpdb->get_row($wpdb->prepare($sql, $match_id));
		
		if ($toprow) {
		
			$sql = "SELECT points, u.display_name, u.ID, p.home_goals, p.away_goals, p.home_penalties, p.away_penalties
					FROM
						{$wpdb->prefix}{$this->prefix}match m,
						{$wpdb->prefix}{$this->prefix}prediction p,
						{$wpdb->prefix}users u
					WHERE
						p.user_id = u.ID AND m.match_id = p.match_id AND m.match_id = %d $filter_user
					ORDER BY
						points DESC
					LIMIT %d";
						
			$result = $wpdb->get_results($wpdb->prepare($sql, $match_id, $limit));
			$output .= "<table class='group zebra'>";
			$output .= "<tr class='row-header'><th class='l col-user'>".__('User', WCP_TD)."</th><th style='white-space:nowrap' class='r'>".$this->team_name($toprow, true).' '. $this->team_score($toprow, true)."</th><th>&ndash;</th><th style='white-space:nowrap' class='l'>".$this->team_score($toprow, false).' '. $this->team_name($toprow, false)."</th><th class='r col-points'>".__('Points', WCP_TD)."</th></tr>";
			foreach ($result as $row) {
				$hpen = '';
				$apen = '';
				if (!$toprow->is_group) { 
					$hpen = '('.$row->home_penalties . ')';
					$apen = '('.$row->away_penalties . ')';
				}
				$output .= "<tr class='row'>";
				$output .= "<td class='l col-user'>".($avatar ? get_avatar( 1, '16', '', $row->display_name ) : '').' '.$row->display_name."</td>";
				$output .= "<td class='r'>$row->home_goals{$hpen}</td>";
				$output .= '<td>&ndash;</td>';
				$output .= "<td class='l'>$row->away_goals{$apen}</td>";
				$output .= "<td class='r col-points'>$row->points</td>";
				$output .= "</tr>";
			}
			$output .= "</table>";
		}
		
		return $output;
	}
	
	function user_predictions() {
		global $wpdb;
		global $current_user;
		
		$output = '';
		
		if (!is_user_logged_in()) {
			return "";
		}
		
		get_currentuserinfo();
		
		$sql = "SELECT is_group,
			h.name AS home_team_name, a.name AS away_team_name,
			p.home_goals, p.away_goals, p.home_penalties, p.away_penalties,
			h.country AS home_country, a.country AS away_country, points
		FROM 
			{$wpdb->prefix}{$this->prefix}match m,
			{$wpdb->prefix}{$this->prefix}prediction p,
			{$wpdb->prefix}{$this->prefix}stage s,
			{$wpdb->prefix}{$this->prefix}team h,
			{$wpdb->prefix}{$this->prefix}team a
		WHERE
			m.stage_id = s.stage_id AND p.match_id = m.match_id AND
			m.home_team_id = h.team_id AND m.away_team_id = a.team_id AND
			p.user_id = %d
		ORDER BY
			sort_order, kickoff";

		$result = $wpdb->get_results( $wpdb->prepare($sql, $current_user->ID) , OBJECT );
		$output .= '<table class="'.$this->prefix.'user_pred_widget">';
		$output .= "<tr>";
		$output .= "<th>".__('Match', WCP_TD)."</th>";
		$output .= "<th>".__('Score', WCP_TD)."</th>";
		$output .= "<th>".__('Pts', WCP_TD)."</th>";
		$output .= "</tr>";
		foreach ($result as $row) {
			$hpen = '';
			$apen = '';
			if (!$row->is_group) { 
				$hpen = '('.$row->home_penalties . ')';
				$apen = '('.$row->away_penalties . ')';
			}
			$output .= "<tr>";
			$output .= "<td>".$this->unclean($row->home_team_name)." ".__('vs',WCP_TD)." ".$this->unclean($row->away_team_name)."</td>";
			$output .= "<td style='text-align:center'>$row->home_goals{$hpen}&nbsp;&ndash&nbsp;$row->away_goals{$apen}</td>";
			$output .= "<td style='text-align:right'>$row->points</td>";
			$output .= "</tr>";
		}
		$output .= "</table>";
		
		return $output;
	}
}
?>