<?php

/**
 * Widgets
 * 
 * Widget class for Wordpress plugin World Cup Predictor
 * 
 * @package WorldCup
 * @version $Id: wcp-widgets.class.php 233381 2010-04-25 09:24:48Z ianhaycox $
 * @author ian
 * Copyright Ian Haycox, 2010
 *
 */
class WorldCupPredictorWidget extends WP_Widget {

	function WorldCupPredictorWidget() {
		$widget_ops = array('classname' => 'widget_'.WCP_TD, 'description' => __('Display World Cup Predictions', WCP_TD) );
		$this->WP_Widget(WCP_TD, __('World Cup Predictor', WCP_TD), $widget_ops);
	}

	function widget($args, $instance) {
		// prints the widget
		extract($args);
		$title = apply_filters('widget_title', $instance['title']);
		$max = $instance['max'];
		$url = $instance['url'];
		$name = $instance['name'];
		$avatar = $instance['avatar'];
		
		echo $before_widget;
		if ( $title )
		echo $before_title . $title . $after_title;

		require_once(dirname(__FILE__).'/wcp-reports.class.php');
		$r = new WorldCupReport();
		
		echo $r->user_ranking($max, $avatar);
		
		if (!empty($url) && !empty($name)) {
			echo '<p><a href="'.$url.'">'.$name.'</a></p>';
		}
				
		echo $after_widget;
	}

	function update($new_instance, $old_instance) {
		//save the widget
		$instance = $old_instance;
		$new_instance = wp_parse_args((array) $new_instance, array( 'title' => ''));
		$instance['title'] = strip_tags($new_instance['title']);
		$new_instance = wp_parse_args((array) $new_instance, array( 'max' => 10));
		$instance['max'] = strip_tags($new_instance['max']);
		$new_instance = wp_parse_args((array) $new_instance, array( 'url' => ''));
		$instance['avatar'] = strip_tags($new_instance['avatar']);
		$new_instance = wp_parse_args((array) $new_instance, array( 'avatar' => 0));
		$instance['url'] = strip_tags($new_instance['url']);
		$new_instance = wp_parse_args((array) $new_instance, array( 'name' => 'Full results'));
		$instance['name'] = strip_tags($new_instance['name']);
		
		return $instance;
	}

	function form($instance) {
		
		global $wpdb;
		
		//widgetform in backend
		$instance = wp_parse_args( (array) $instance, array( 'title' => '', 'max' => 10, 'avatar' => 0, 'url' => '', 'name' => 'Full results') );
		$title = $instance['title'];
		$max = $instance['max'];
		if (!is_numeric($max)) $max = 10;
		$avatar = $instance['avatar'];
		$url = $instance['url'];
		$name = $instance['name'];
		
?>
		<p><?php _e('Display Rankings.', WCP_TD); ?></p>
		<p><label for="<?php echo $this->get_field_id('title'); ?>"><?php _e('Title:', WCP_TD); ?>
		<input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo esc_attr($title); ?>" /></label></p>
		
		<p><label for="<?php echo $this->get_field_id('max'); ?>"><?php _e('Max rankings to show:', WCP_TD); ?>
		<input class="widefat" id="<?php echo $this->get_field_id('max'); ?>" name="<?php echo $this->get_field_name('max'); ?>" type="text" value="<?php echo esc_attr($max); ?>" /></label></p>
		
		<p><label for="<?php echo $this->get_field_id('avatar'); ?>"><?php _e('Show Avatars:', WCP_TD); ?>
		<input class="widefat" id="<?php echo $this->get_field_id('avatar'); ?>" name="<?php echo $this->get_field_name('avatar'); ?>" type="checkbox" value="1" <?php echo $avatar ? ' checked ' : ''; ?>" /></label></p>
		
		<p><label for="<?php echo $this->get_field_id('url'); ?>"><?php _e('Full rankings page URL:', WCP_TD); ?>
		<input class="widefat" id="<?php echo $this->get_field_id('url'); ?>" name="<?php echo $this->get_field_name('url'); ?>" type="text" value="<?php echo esc_attr($url); ?>" /></label></p>
		
		<p><label for="<?php echo $this->get_field_id('name'); ?>"><?php _e('Full rankings link name:', WCP_TD); ?>
		<input class="widefat" id="<?php echo $this->get_field_id('name'); ?>" name="<?php echo $this->get_field_name('name'); ?>" type="text" value="<?php echo esc_attr($name); ?>" /></label></p>

<?php
	}


}

class WorldCupPredictorUserWidget extends WP_Widget {

	function WorldCupPredictorUserWidget() {
		$widget_ops = array('classname' => 'widget_user_'.WCP_TD, 'description' => __('Display Users World Cup Predictions', WCP_TD) );
		$this->WP_Widget(WCP_TD.'user', __('World Cup User Predictions', WCP_TD), $widget_ops);
	}

	function widget($args, $instance) {
		// prints the widget
		extract($args);
		$title = apply_filters('widget_title', $instance['title']);
		
		echo $before_widget;
		if ( $title )
		echo $before_title . $title . $after_title;

		require_once(dirname(__FILE__).'/wcp-reports.class.php');
		$r = new WorldCupReport();
		
		echo $r->user_predictions();
		
		echo $after_widget;
	}

	function update($new_instance, $old_instance) {
		//save the widget
		$instance = $old_instance;
		$new_instance = wp_parse_args((array) $new_instance, array( 'title' => ''));
		$instance['title'] = strip_tags($new_instance['title']);
		return $instance;
	}

	function form($instance) {
		
		global $wpdb;
		
		//widgetform in backend
		$instance = wp_parse_args( (array) $instance, array( 'title' => '') );
		$title = $instance['title'];
		
?>
		<p><?php _e('Display User Predictions.', WCP_TD); ?></p>
		<p><label for="<?php echo $this->get_field_id('title'); ?>"><?php _e('Title:', WCP_TD); ?>
		<input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo esc_attr($title); ?>" /></label></p>
		
<?php
	}


}
