<?php 
/*
Plugin Name: World Cup Predictor
Plugin URI: http://www.ianhaycox.com/worldcuppredictor
Description: World Cup Predictor - Guess the outcome of World Cup football/soccer games, aka Fantasy Football
Version: 0.4.1
SVN Version: $Id: world-cup-predictor.php 236599 2010-05-02 14:29:16Z ianhaycox $
Author: Ian Haycox
Author URI: http://www.ianhaycox.com
Copyright Ian Haycox, 2010 (email : ian.haycox@gmail.com)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/**
 * @package WorldCup
 * @version $Id: world-cup-predictor.php 236599 2010-05-02 14:29:16Z ianhaycox $
 * @author ian
 * Copyright Ian Haycox, 2010
 *
 */

define('WCP_TD', 'world-cup-predictor');  // Text domain & plugin dir
load_plugin_textdomain(WCP_TD, 'wp-content/plugins/'.WCP_TD.'/lang');
require_once(dirname(__FILE__).'/wcp-predict.class.php');

class WorldCup {
	
	const VERSION = '0.4.1';

	var $prefix = 'wcp2010_';

	/**
	 * error handling
	 *
	 * @var boolean
	 */
	private $error = false;
	
	/**
	 * message
	 *
	 * @var string
	 */
	private $message = null;
	
	function __construct() {
	}
		
	function print_styles() {
?>		
<!-- Added World Cup Predictor Plugin <?php echo self::VERSION; ?> by Ian Haycox. -->
<link type="text/css" rel="stylesheet" href="<?php echo WP_PLUGIN_URL . '/' . WCP_TD; ?>/css/style.css" />
<link type="text/css" rel="stylesheet" href="<?php echo WP_PLUGIN_URL . '/' . WCP_TD; ?>/css/jquery.countdown.css" />
<script type="text/javascript">
//<![CDATA[
	WorldCupPredictorAjax = { blogUrl: "<?php bloginfo( 'wpurl' ); ?>" };
//]]>
</script>
<?php		
	}
	
	function print_scripts() {
		wp_enqueue_script($this->prefix . 'js', WP_PLUGIN_URL . '/' . WCP_TD.'/js/wcp.js', array( 'jquery', 'jquery-ui-dialog'));
		wp_enqueue_script($this->prefix . 'countdown_js', WP_PLUGIN_URL . '/' . WCP_TD.'/js/jquery.countdown.min.js', array( 'jquery', 'jquery-ui-dialog'));
	}
	
	/**
	 * Initialize the plugin widgets
	 */
	function widgets_init() {
		/*
		 * For the results table
		 */
		require_once(dirname(__FILE__).'/wcp-widgets.class.php');
		register_widget('WorldCupPredictorWidget');
		register_widget('WorldCupPredictorUserWidget');
	}
	
	function wp_footer() {
		if (get_option($this->prefix.'promo_link', 1)) {
			echo '<p><a href="http://www.ianhaycox.com/">World Cup Predictor by Ian Haycox</p>';
		}
	}
	
	/*
	 * Process shortcode [world-cup-predictor]
	 * 
	 */
	function shortcode($atts) {
		extract(shortcode_atts(array(
			'predict' => 1,
			'ranking' => 0,
			'tables' => 0,		// Show group tables
			'scores' => 0,		// All users predictions by match id
			'stage' => 0,		// group id - zero = all
			'show_results' => 1,// show match results below group tables
			'results' => 0,		// match results
			'knockout' => 0,	// Knockout stage results
			'user' => 0,		// Display current users predictions
			'limit' => 999999,	// Limit ranking and prediction scores
		), $atts));

		$output = '';
		
		if (!is_numeric($stage)) {
			$stage = 0;
		}
		
		if (!is_numeric($limit)) {
			$limit = 999999;
		}
		
		if ($ranking) {
			require_once(dirname(__FILE__).'/wcp-reports.class.php');
			$r = new WorldCupReport();
			$output =  $r->user_ranking($limit);
			return $output;
		}
		
		if ($tables) {
			require_once(dirname(__FILE__).'/wcp-reports.class.php');
			$r = new WorldCupReport();
			$output =  $r->group_tables($stage, $show_results);
			return $output;
		}
		
		if ($scores) {
			require_once(dirname(__FILE__).'/wcp-reports.class.php');
			$r = new WorldCupReport();
			$output =  $r->user_scores($scores, $limit);
			return $output;
		}
		
		if ($results) {
			require_once(dirname(__FILE__).'/wcp-reports.class.php');
			$r = new WorldCupReport();
			$output =  $r->results($stage);
			return $output;
		}
		
		if ($knockout) {
			require_once(dirname(__FILE__).'/wcp-reports.class.php');
			$r = new WorldCupReport();
			$output =  $r->knockout();
			return $output;
		}
		
		if ($user) {
			require_once(dirname(__FILE__).'/wcp-reports.class.php');
			$r = new WorldCupReport();
			$output =  $r->user_predictions();
			return $output;
		}
		
		if ($predict) {
			require_once(dirname(__FILE__).'/wcp-predict.class.php');
			$p = new WorldCupPredict();
			$output =  $p->prediction_form($stage);
			return $output;
		}
		
	}
	
	function debug($var, $echo = true) {
		$output = "<pre>";
		$output .= print_r($var, true);
		$output .= "</pre>";
		if ($echo) echo $output;
		return $output;
	}
	
	/**
	 * Clean the input string of dangerous input.
	 * @param $str input string
	 * @return cleaned string.
	 */
	function clean($str) {
		$str = strip_tags($str);
		return @trim(htmlspecialchars($str, ENT_QUOTES));
	}
	
	/**
	 * Reverse clean() after getting from DB
	 * @param $str input string
	 * @return cleaned string.
	 */
	function unclean($str) {
		return stripslashes($str);
	}
	
	function flag($country) {
		return '<img class="'.$this->prefix.'flag" src="'.WP_PLUGIN_URL.'/'.WCP_TD.'/images/'.strtolower($country).'.png" />';
	}
	
	/**
	 * set message
	 *
	 * @param string $message
	 * @param boolean $error triggers error message if true
	 * @return none
	 */
	function setMessage( $message, $error = false )
	{
		$type = 'success';
		if ( $error ) {
			$this->error = true;
			$type = 'error';
		}
		$this->message[$type] = $message;
	}
	
	
	/**
	 * return message
	 *
	 * @param none
	 * @return string
	 */
	function getMessage()
	{
		if (is_null($this->message) || (empty($this->message))) return false;
		
		if ( $this->error )
			return $this->message['error'];
		else
			return $this->message['success'];
	}
	
	
	/**
	 * print formatted message
	 *
	 * @param none
	 * @return string
	 */
	function printMessage($echo = true)
	{
		if ($this->getMessage() === false)  return '';
		
		$str = '';
		
		if ( $this->error )
			$str = "<div class='message error'><p>".$this->getMessage()."</p></div>";
		else
			$str = "<div class='message updated fade'><p><strong>".$this->getMessage()."</strong></p></div";
		$this->message = null;
		
		if (!$echo) return $str;
		echo $str;
	}
	
	/*
	 * Return an href
	 */
	function mklink($str, $url, $title) {
		$link = $str;
		if (!empty($url)) {
			$link = '<a href="'.$url.'" title="'.$title.'" alt="'.$title.'">'.$str.'</a>';
		} else {
			$link = '<span title="'.$title.'">'.$str.'</span>';
		}
		return $link;
	}
	
}

$wcp = new WorldCup();

add_shortcode(WCP_TD, array(&$wcp, 'shortcode'));

add_action('wp_print_styles', array(&$wcp, 'print_styles'));
add_action('wp_print_scripts', array(&$wcp, 'print_scripts'));
add_action('widgets_init', array(&$wcp, 'widgets_init'));
add_action('wp_footer', array(&$wcp, 'wp_footer'));

$wcpp = new WorldCupPredict();
add_action('wp_ajax_worldcuppredictor_ajax', array(&$wcpp,'ajax'));


if (is_admin()) {

	require_once(dirname(__FILE__).'/admin/wcp-admin.class.php');
	require_once(dirname(__FILE__).'/admin/wcp-teams.class.php');
	require_once(dirname(__FILE__).'/admin/wcp-venues.class.php');
	require_once(dirname(__FILE__).'/admin/wcp-stages.class.php');
	require_once(dirname(__FILE__).'/admin/wcp-matches.class.php');
	require_once(dirname(__FILE__).'/admin/wcp-predictions.class.php');
	require_once(dirname(__FILE__).'/admin/wcp-scoring.class.php');
	require_once(dirname(__FILE__).'/admin/wcp-overview.class.php');
	require_once(dirname(__FILE__).'/admin/wcp-results.class.php');
	require_once(dirname(__FILE__).'/wcp-reports.class.php');

	
	$wcpadmin = new WorldCupAdmin();

	// Activation function
	register_activation_hook(__FILE__, array(&$wcpadmin, 'activate'));
	register_deactivation_hook(__FILE__, array(&$wcpadmin, 'deactivate'));

	// Register a uninstall hook to automatically remove all tables & option
	if ( function_exists('register_uninstall_hook') )
		register_uninstall_hook( __FILE__ , array(&$wcpadmin, 'uninstall') );

	add_action('admin_menu', array(&$wcpadmin, 'admin_menu'));
	add_action('admin_print_scripts', array(&$wcpadmin, 'admin_print_scripts'));
	add_action('admin_print_styles', array(&$wcpadmin, 'admin_print_styles'));
	add_action('admin_init', array(&$wcpadmin, 'admin_init'));
}


?>